import { __decorate, __param } from 'vscode/external/tslib/tslib.es6.js';
import { CancellationToken } from 'vscode/vscode/vs/base/common/cancellation';
import { CancellationError } from 'vscode/vscode/vs/base/common/errors';
import { Disposable } from 'vscode/vscode/vs/base/common/lifecycle';
import { equals } from 'vscode/vscode/vs/base/common/objects';
import { localizeWithPath } from 'vscode/vscode/vs/nls';
import { IDialogService } from 'vscode/vscode/vs/platform/dialogs/common/dialogs.service';
import { ILogService } from 'vscode/vscode/vs/platform/log/common/log.service';
import { IProductService } from 'vscode/vscode/vs/platform/product/common/productService.service';
import { asJson } from 'vscode/vscode/vs/platform/request/common/request';
import { IRequestService } from 'vscode/vscode/vs/platform/request/common/request.service';
import { ITelemetryService } from 'vscode/vscode/vs/platform/telemetry/common/telemetry.service';
import { IUserDataProfilesService } from 'vscode/vscode/vs/platform/userDataProfile/common/userDataProfile.service';
import { toWorkspaceIdentifier } from 'vscode/vscode/vs/platform/workspace/common/workspace';
import { IWorkspaceContextService } from 'vscode/vscode/vs/platform/workspace/common/workspace.service';
import { IWorkbenchEnvironmentService } from 'vscode/vscode/vs/workbench/services/environment/common/environmentService.service';
import { IExtensionService } from 'vscode/vscode/vs/workbench/services/extensions/common/extensions.service';
import { IHostService } from 'vscode/vscode/vs/workbench/services/host/browser/host.service';
import { IUserDataProfileService } from 'vscode/vscode/vs/workbench/services/userDataProfile/common/userDataProfile.service';

let UserDataProfileManagementService = class UserDataProfileManagementService extends Disposable {
    constructor(userDataProfilesService, userDataProfileService, hostService, dialogService, workspaceContextService, extensionService, environmentService, telemetryService, productService, requestService, logService) {
        super();
        this.userDataProfilesService = userDataProfilesService;
        this.userDataProfileService = userDataProfileService;
        this.hostService = hostService;
        this.dialogService = dialogService;
        this.workspaceContextService = workspaceContextService;
        this.extensionService = extensionService;
        this.environmentService = environmentService;
        this.telemetryService = telemetryService;
        this.productService = productService;
        this.requestService = requestService;
        this.logService = logService;
        this._register(userDataProfilesService.onDidChangeProfiles(e => this.onDidChangeProfiles(e)));
        this._register(userDataProfilesService.onDidResetWorkspaces(() => this.onDidResetWorkspaces()));
        this._register(userDataProfileService.onDidChangeCurrentProfile(e => this.onDidChangeCurrentProfile(e)));
        this._register(userDataProfilesService.onDidChangeProfiles(e => {
            const updatedCurrentProfile = e.updated.find(p => this.userDataProfileService.currentProfile.id === p.id);
            if (updatedCurrentProfile) {
                this.changeCurrentProfile(updatedCurrentProfile, ( localizeWithPath(
                    'vs/workbench/services/userDataProfile/browser/userDataProfileManagement',
                    'reload message when updated',
                    "The current profile has been updated. Please reload to switch back to the updated profile"
                )));
            }
        }));
    }
    onDidChangeProfiles(e) {
        if (( e.removed.some(profile => profile.id === this.userDataProfileService.currentProfile.id))) {
            this.changeCurrentProfile(this.userDataProfilesService.defaultProfile, ( localizeWithPath(
                'vs/workbench/services/userDataProfile/browser/userDataProfileManagement',
                'reload message when removed',
                "The current profile has been removed. Please reload to switch back to default profile"
            )));
            return;
        }
    }
    onDidResetWorkspaces() {
        if (!this.userDataProfileService.currentProfile.isDefault) {
            this.changeCurrentProfile(this.userDataProfilesService.defaultProfile, ( localizeWithPath(
                'vs/workbench/services/userDataProfile/browser/userDataProfileManagement',
                'reload message when removed',
                "The current profile has been removed. Please reload to switch back to default profile"
            )));
            return;
        }
    }
    async onDidChangeCurrentProfile(e) {
        if (e.previous.isTransient) {
            await this.userDataProfilesService.cleanUpTransientProfiles();
        }
    }
    async createAndEnterProfile(name, options) {
        const profile = await this.userDataProfilesService.createNamedProfile(name, options, toWorkspaceIdentifier(this.workspaceContextService.getWorkspace()));
        await this.changeCurrentProfile(profile);
        this.telemetryService.publicLog2('profileManagementActionExecuted', { id: 'createAndEnterProfile' });
        return profile;
    }
    async createAndEnterTransientProfile() {
        const profile = await this.userDataProfilesService.createTransientProfile(toWorkspaceIdentifier(this.workspaceContextService.getWorkspace()));
        await this.changeCurrentProfile(profile);
        this.telemetryService.publicLog2('profileManagementActionExecuted', { id: 'createAndEnterTransientProfile' });
        return profile;
    }
    async updateProfile(profile, updateOptions) {
        if (!( this.userDataProfilesService.profiles.some(p => p.id === profile.id))) {
            throw new Error(`Profile ${profile.name} does not exist`);
        }
        if (profile.isDefault) {
            throw new Error(localizeWithPath('vs/workbench/services/userDataProfile/browser/userDataProfileManagement', 'cannotRenameDefaultProfile', "Cannot rename the default profile"));
        }
        await this.userDataProfilesService.updateProfile(profile, updateOptions);
        this.telemetryService.publicLog2('profileManagementActionExecuted', { id: 'updateProfile' });
    }
    async removeProfile(profile) {
        if (!( this.userDataProfilesService.profiles.some(p => p.id === profile.id))) {
            throw new Error(`Profile ${profile.name} does not exist`);
        }
        if (profile.isDefault) {
            throw new Error(localizeWithPath('vs/workbench/services/userDataProfile/browser/userDataProfileManagement', 'cannotDeleteDefaultProfile', "Cannot delete the default profile"));
        }
        await this.userDataProfilesService.removeProfile(profile);
        this.telemetryService.publicLog2('profileManagementActionExecuted', { id: 'removeProfile' });
    }
    async switchProfile(profile) {
        const workspaceIdentifier = toWorkspaceIdentifier(this.workspaceContextService.getWorkspace());
        if (!( this.userDataProfilesService.profiles.some(p => p.id === profile.id))) {
            throw new Error(`Profile ${profile.name} does not exist`);
        }
        if (this.userDataProfileService.currentProfile.id === profile.id) {
            return;
        }
        await this.userDataProfilesService.setProfileForWorkspace(workspaceIdentifier, profile);
        await this.changeCurrentProfile(profile);
        this.telemetryService.publicLog2('profileManagementActionExecuted', { id: 'switchProfile' });
    }
    async getBuiltinProfileTemplates() {
        if (this.productService.profileTemplatesUrl) {
            try {
                const context = await this.requestService.request({ type: 'GET', url: this.productService.profileTemplatesUrl }, CancellationToken.None);
                if (context.res.statusCode === 200) {
                    return (await asJson(context)) || [];
                }
                else {
                    this.logService.error('Could not get profile templates.', context.res.statusCode);
                }
            }
            catch (error) {
                this.logService.error(error);
            }
        }
        return [];
    }
    async changeCurrentProfile(profile, reloadMessage) {
        const isRemoteWindow = !!this.environmentService.remoteAuthority;
        const shouldRestartExtensionHosts = this.userDataProfileService.currentProfile.id !== profile.id || !equals(this.userDataProfileService.currentProfile.useDefaultFlags, profile.useDefaultFlags);
        if (shouldRestartExtensionHosts) {
            if (!isRemoteWindow) {
                if (!(await this.extensionService.stopExtensionHosts(( localizeWithPath(
                    'vs/workbench/services/userDataProfile/browser/userDataProfileManagement',
                    'switch profile',
                    "Switching to a profile."
                ))))) {
                    if (( this.userDataProfilesService.profiles.some(p => p.id === this.userDataProfileService.currentProfile.id))) {
                        await this.userDataProfilesService.setProfileForWorkspace(toWorkspaceIdentifier(this.workspaceContextService.getWorkspace()), this.userDataProfileService.currentProfile);
                    }
                    throw new CancellationError();
                }
            }
        }
        await this.userDataProfileService.updateCurrentProfile(profile);
        if (shouldRestartExtensionHosts) {
            if (isRemoteWindow) {
                const { confirmed } = await this.dialogService.confirm({
                    message: reloadMessage ?? ( localizeWithPath(
                        'vs/workbench/services/userDataProfile/browser/userDataProfileManagement',
                        'reload message',
                        "Switching a profile requires reloading VS Code."
                    )),
                    primaryButton: ( localizeWithPath(
                        'vs/workbench/services/userDataProfile/browser/userDataProfileManagement',
                        'reload button',
                        "&&Reload"
                    )),
                });
                if (confirmed) {
                    await this.hostService.reload();
                }
            }
            else {
                await this.extensionService.startExtensionHosts();
            }
        }
    }
};
UserDataProfileManagementService = ( __decorate([
    ( __param(0, IUserDataProfilesService)),
    ( __param(1, IUserDataProfileService)),
    ( __param(2, IHostService)),
    ( __param(3, IDialogService)),
    ( __param(4, IWorkspaceContextService)),
    ( __param(5, IExtensionService)),
    ( __param(6, IWorkbenchEnvironmentService)),
    ( __param(7, ITelemetryService)),
    ( __param(8, IProductService)),
    ( __param(9, IRequestService)),
    ( __param(10, ILogService))
], UserDataProfileManagementService));

export { UserDataProfileManagementService };
